(* $Id: godi_file_handler.ml 711 2007-07-30 22:37:57Z gerd $ *)

open Godi_types

let scheme_re = Str.regexp "^\\([a-zA-Z]+\\)://[^/]+/"

let get_scheme name =
  (* Return the scheme or raise Not_found *)
  if Str.string_match scheme_re name 0 then
    Str.matched_group 1 name
  else
    raise Not_found

let is_local name =
  try 
    let _sch = get_scheme name in false
  with Not_found -> true

let name_suff_re = Str.regexp "^\\(.*/\\)\\([^/]+\\)/?$"

let split_name name =
  if is_local name then
    (Filename.dirname name, Filename.basename name)
  else
    try
      if Str.string_match name_suff_re name 0 then (
	let pref = Str.matched_group 1 name in
	let suff = Str.matched_group 2 name in
	if is_local pref then raise Not_found;
	(pref, suff)
      )
      else
	raise Not_found
    with
      | Not_found ->
	  raise(File_handler_error("Cannot determine directory of " ^ name))

let file_handler name : file_handler =
  if Godi_file.file_is_dir name then (
    object
      method url = name
      method list con () =
	Godi_file.read_dir name
      method get con filename =
	(Filename.concat name filename, None)
    end
  )
  else
    raise(File_handler_error("Not a directory: " ^ name))


let reg = Hashtbl.create 10

let handler_of_directory name =
  try
    let scheme = get_scheme name in
    (Hashtbl.find reg scheme) name
  with
    | Not_found -> file_handler name

let register scheme handler_factory =
  Hashtbl.replace reg scheme handler_factory
