(* $Id: godi_getopt.ml 744 2008-05-19 23:05:11Z gerd $ *)

type action =
  [ `Set of bool ref
  | `Clear of bool ref
  | `Unit of unit -> unit
  | `String of string -> unit
  | `Set_string of string ref
  | `End_of_switches
  | `Help
  ]


type spec = string * action * string


let has_arg =
  function
    | `Set _ | `Clear _ | `Unit _ | `End_of_switches | `Help -> false
    | `String _ | `Set_string _ -> true


let usage spec help =
  let buf = Buffer.create 100 in
  Buffer.add_string buf help;
  Buffer.add_string buf "\n\n";
  List.iter
    (fun (key, _, comment) ->
       Buffer.add_string buf (key ^ "  " ^ comment ^ "\n");
    )
    spec;
  Buffer.contents buf


let parse spec anon_fun help =
  try
    let eos = ref false in
    while !Arg.current < Array.length Sys.argv - 1 do
      incr Arg.current;
      let opt = Sys.argv.( !Arg.current ) in
      if not !eos && opt <> "" && opt.[0] = '-' then (
	let l = String.length opt in
	for k = 1 to l-1 do
	  let c = opt.[k] in
	  let key = "-" ^ String.make 1 c in
	  let _,action,_ =
	    try List.find (fun (k,_,_) -> k=key) spec
	    with
	      | Not_found -> raise(Arg.Bad ("Unknown option: " ^ key)) in
	  if has_arg action then (
	    if k <> l-1 then
	      raise(Arg.Bad("This option style is not supported for " ^ key));
	    if !Arg.current >= Array.length Sys.argv - 1 then
	      raise(Arg.Bad("Argument missing for " ^ key));
	    incr Arg.current;
	    let arg = Sys.argv.( !Arg.current ) in
	    match action with
	      | `String f -> f arg
	      | `Set_string r -> r := arg
	      | _ -> ()
	  )
	  else
	    match action with
	      | `Set r -> r := true
	      | `Clear r -> r := false
	      | `Unit f -> f()
	      | `End_of_switches -> eos := true
	      | `Help -> raise (Arg.Help (usage spec help))
	      | _ -> ()
	done
      )
      else
	anon_fun opt
    done
  with
    | Arg.Bad msg ->
	prerr_endline msg;
	exit 1

    | Arg.Help msg ->
	print_endline msg;
	exit 0

